@@ -1,3 +1,4 @@
+#include <unistd.h>
 
 #include "hd-dbus.h"
 #include "hd-switcher.h"
@@ -21,6 +22,7 @@
 #define DSME_SHUTDOWN_SIGNAL_NAME "shutdown_ind"
 
 gboolean hd_dbus_display_is_off = FALSE;
+gboolean hd_dbus_tklock_on = FALSE;
 
 static DBusConnection *connection, *sysbus_conn;
 
@@ -56,14 +58,27 @@ hd_dbus_system_bus_signal_handler (DBusConnection *conn,
                                    DBusMessage *msg, void *data)
 {
   HdCompMgr  * hmgr = data;
+  extern MBWindowManager *hd_mb_wm;
+
   if (dbus_message_is_signal(msg, DSME_SIGNAL_INTERFACE,
 			     DSME_SHUTDOWN_SIGNAL_NAME))
     {
+      Window overlay;
       g_warning ("%s: " DSME_SHUTDOWN_SIGNAL_NAME " from DSME", __func__);
-      /* send TERM to applications and exit */
+      /* send TERM to applications and exit without cleanup */
       hd_volume_profile_set_silent (TRUE);
       hd_comp_mgr_kill_all_apps (hmgr);
-      exit (0);
+      overlay = mb_wm_comp_mgr_clutter_get_overlay_window (
+                        MB_WM_COMP_MGR_CLUTTER (hmgr));
+      if (overlay != None)
+        {
+          /* needed because of the non-composite optimisations in X,
+           * otherwise we could show garbage if the shutdown screen is
+           * a bit slow or missing */
+          XClearWindow (hd_mb_wm->xdpy, overlay);
+          XFlush (hd_mb_wm->xdpy);
+        }
+      _exit (0);
     }
   else if (dbus_message_is_signal (msg, MCE_SIGNAL_IF, "tklock_mode_ind"))
     {
@@ -71,7 +86,21 @@ hd_dbus_system_bus_signal_handler (DBusConnection *conn,
       if (dbus_message_get_args (msg, NULL, DBUS_TYPE_STRING, &mode,
                                  DBUS_TYPE_INVALID));
         {
-          hd_comp_mgr_tklocked (hmgr, !strcmp(mode, MCE_TK_LOCKED));
+          if (strcmp(mode, MCE_TK_LOCKED))
+            {
+              if (hd_dbus_tklock_on)
+                {
+                  hd_dbus_tklock_on = FALSE;
+                  /* if we avoided focusing a window during tklock, do it now
+                   * (this only has an effect if no window is currently
+                   * focused) */
+                  mb_wm_unfocus_client (hd_mb_wm, NULL);
+                }
+            }
+          else
+            {
+              hd_dbus_tklock_on = TRUE;
+            }
         }
     }
   else if (dbus_message_is_signal (msg, MCE_SIGNAL_IF, "display_status_ind"))
